#include <VRS51L3074_SDCC.h>


#define RXBUF0SZ 64

__xdata unsigned char ser0rxbuf[RXBUF0SZ];
static __xdata unsigned char *txptr,*rxptr;
unsigned char ser0txbytes,ser0rxbytes;

/*
   Rx is always enabled, flush the buffer with ser0flush(), check
   available chars watching ser0rxbytes, peek last received char
   calling ser0peek(). Data is at ser0rxbuf in xram
   
   Enable Tx by calling ser0send with a pointer to data in xram
   and the number of bytes to send. Check if finished by watching
   ser0txbytes, should be 0 when finished (or check UART0INT bit 4, BTW).
*/

void ser0Interrupt(void) interrupt 5
{
register unsigned char st,dat;
  
    st=UART0INT;
    if(st&0x6){                // RXAVENF or RXOVF
      dat=UART0BUF;             // read it 
      if(ser0rxbytes<RXBUF0SZ){     // avoid buffer overflow
	*(rxptr++)=dat;             // put it in the buffer
        if(st&(1<<2))
	  rxptr--;                      // ignore on RxOV int (but read anyway)
        else
	  ser0rxbytes++;
      }
    }
    else {                           // TXEMPTYF
      if(ser0txbytes){
	UART0BUF=*(txptr++);           // if bytes, send them
	--ser0txbytes;
      }
      if(!ser0txbytes)
	UART0INT=0x62;           // if no more bytes, disable interrupt
    }
}

// Returns -1 if there is no char available
int ser0peek(void)
{
  if(ser0rxbytes)
    return(*(rxptr-1));
  else 
    return(-1);
}


void ser0flush(void)
{
register unsigned char aux;

  aux=UART0INT;
  aux&=0x70;                 // mask out non interrupt bits
  aux|=0x02;                  // keep reception enabled
  UART0INT=(aux&~0x60);      // disable RXAVENF + RXOVF interrupts
  rxptr=ser0rxbuf;
  ser0rxbytes=0;
  UART0INT=aux;               // restore interrupts
}

void ser0send(__xdata unsigned char *ptr,unsigned char bytes)
{
  UART0INT=0x62;   	     // disable TXEMPTY interrupt
  txptr=ptr;
  ser0txbytes=bytes;
  UART0INT=0x72;  	    // enable TXEMPTY interrupt
}

void ser0init(unsigned long clockspeed)
{
register unsigned char aux;
register unsigned int aux2;
  
	PERIPHEN1 |= (1<<3);			// Enable UART0

	aux=(unsigned char)(clockspeed>>1)&0x0F; // Fine adjustment on baud rate
	UART0CFG = (aux<<4);			//Use internal clock
						//9th bit not used
						//only one stop bit

	UART0INT = 0x62;			//Enable RXAV + RXOV int + Enable Reception
	UART0EXT = 0x00;			//Not using UART0 Extensions

	aux2=((unsigned int)(clockspeed>>5)&0xFFFF)-1;   // BRG = (clock/speed)/32 -1
	UART0BRL = (unsigned char)aux2&0xFF;
	UART0BRH = (unsigned char)(aux2>>8)&0xFF;
	ser0flush();
	INTSRC1 &= ~(1<<5);                   // UART0 module interrupt
	INTEN1 |= (1<<5);                     // interrupt 5 enable
}

